#ifndef MEDIA_LIBRARY_H
#define MEDIA_LIBRARY_H

/* Initial size of playlists array within the media_library struct,
 * and how much to allocate when a resize is needed
 */
#define ML_PLAYLISTS_START_SIZE  100
#define ML_PLAYLISTS_CHUNK_SIZE  50


/* some magic numbers... the library and any search results are playlists
 * within the media library's list of playlists.  they occur at these
 * fixed indices.
 */
#define ML_NUM_PSEUDO_PLAYLISTS  2
#define ML_PINDEX_LIBRARY        0
#define ML_PINDEX_SRESULTS       1

#include "meta_info.h"
#include "meta_info_db.h"
#include "playlist.h"
#include "shameful_limits.h"

typedef struct
{
   /* some locations of where things are loaded/saved */
   char     *midb_file;       /* file containing the database */
   char     *playlist_dir;    /* directory where playlists are stored */

   /* the actual meta-information database */
   meta_info_db   *midb;

   /* the playlists */
   playlist **playlists;
   int        playlists_capacity;   /* total size of playlists array */
   int        playlists_num;        /* how many of the elements are used */
      /* The playlists array is implemented as a vector, not a linked list.
       * When first created, it is sized to ML_PLAYLISTS_START_SIZE (see
       * above).  When that capacity is reached, it is increased by
       * ML_PLAYLISTS_CHUNK_SIZE (also above).
       */

} media_library;


/* functions to allocate, free, and initialize a media database */
media_library *ml_new(void);
void ml_free(media_library *ml);

/* functions to add/remove and search playlists in a media library */
void ml_add_playlist(media_library *ml, playlist *p);
void ml_remove_playlist(media_library *ml, int pindex);
void ml_search_playlist(media_library *ml, int pindex, char *query);

/* function to load a media library from the provided files */
void ml_load(media_library *ml, const char *dbfile, const char *pdir);

#endif
