#ifndef MEDIA_PLAYER_H
#define MEDIA_PLAYER_H

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <err.h>

#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>

#include "playlist.h"

/* structure used to track status of media player */
typedef struct
{
   bool  is_playing;  /* flag indicating if media player is playing */
   float position;    /* position, in seconds, into currently playing file */
} mp_status_t;


/* global status object used to track the status of the media player */
extern mp_status_t mp_status;


/* enumeration of available play-modes.  included are:
 *    Linear:  Songs in the queue play in the order they appear
 *    Loop:    Like linear, but when the end is reached, the queue restarts
 *    Random:  Songs are chosen at random and play never ends
 */
typedef enum
{  MP_MODE_LINEAR,
   MP_MODE_LOOP,
   MP_MODE_RANDOM
} playmode;

/* main structure encapsulating info about the media player itself */
typedef struct
{
   /* info about the forked media player */
   pid_t    pid;           /* process id */
   int      pipe_read;     /* read-end of pipe */
   int      pipe_write;    /* write-end of pipe */

   /* currently playing queue information */
   playlist *queue;     /* pointer to size of queue */
   int       current;   /* index into above playlist of current item */

   /* flags used to track state of the media player */
   bool     should_be_playing;
   bool     paused;
   bool     just_started_playing;

   /* misc info about the player */
   playmode mode;
   int      rand_seed;

} media_player;


/* media player create/destroy functions */
void media_player_init(media_player *mp);
void media_player_destroy(media_player *mp);


/* media player control functions */
void media_player_play(media_player *mp);
void media_player_stop(media_player *mp);
void media_player_pause(media_player *mp);
void media_player_unpause(media_player *mp);
void media_player_seek(media_player *mp, int seconds);
void media_player_set_queue(media_player *mp, playlist *queue);

/* monitor function */
void media_player_monitor(media_player *mp);


#endif
