#ifndef META_INFO_DB
#define META_INFO_DB

#include <sys/errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fts.h>

#include "meta_info.h"
#include "shameful_limits.h"


/* The main struct of this module, containing all records of meta info. for
 * every media file in the db.
 * It's simply an array of meta info structure.  It exists as it's own
 * module to simply wrap all the functionality we need with the MIDB into
 * one file.
 */
typedef struct {

   int          num_files;
   meta_info   *files[MAX_NUM_SONGS_PER_PLAYLIST];

} meta_info_db;


/* functions to create/destory meta_info_db structs */
meta_info_db *midb_new(void);
void midb_free(meta_info_db *midb);

/* functions to load/save databases from/to a file */
void midb_load(const char *dbfile, meta_info_db *midb);
void midb_save(const char *dbfile, const meta_info_db *midb);

/* function to dump the contents of a midb in an easily parsable format */
void midb_flush(const meta_info_db *midb, FILE *out);

/* functions to add/remove records to/from a database */
void midb_add_record(meta_info_db *midb, meta_info *file);
void midb_remove_record(meta_info_db *midb, int index);
void midb_update_record(meta_info_db *midb, int index, meta_info *newRecord);

/* query methods */
int midb_query_by_filename(const meta_info_db *midb, const char *filename);

/* functions to create/update a database stored in a given file */
void midb_create(const char *dbfile, char *dirlist[]);
void midb_update(const char *dbfile);

#endif
