#ifndef PLAYLIST_H
#define PLAYLIST_H

#include <sys/types.h>
#include <sys/stat.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glob.h>
#include <unistd.h>
#include <libgen.h>
#include <errno.h>
#include <err.h>

#include "meta_info.h"
#include "meta_info_db.h"
#include "shameful_limits.h"


/* the core playlist structure */
typedef struct
{
   char  *filename;        /* filename containing the playlist */
   char  *name;            /* the name of the playlist used in display */
   int    num_files;       /* the number of songs in the playlist */

   /* the array of files (their meta information) in the playlist */
   meta_info   *files[MAX_NUM_SONGS_PER_PLAYLIST];

} playlist;

/* IMPORTANT NOTES ABOUT THE "playlist" STRUCTURE:
 * 1. The "meta" array is an array of the meta information corresponding
 *    to each song in the playlist.
 *
 * 2. The elements of the "meta" array are simply pointers to the
 *    already existing meta-info elements in the meta-info-database.
 *
 * 3. When loading a playlist from a file, each element of the playlist
 *    is compared against the meta information database to find a
 *    corresponding entry.  If no such file exists in the meta info
 *    databaes, a new record is added to the DB but it only contains the
 *    filename read from the playlist file (no meta info).
 */

/* functions to allocate/free/duplicate playlist structs */
playlist *playlist_new(void);
void playlist_free(playlist *p);
playlist *playlist_dup(const playlist *original, const char *filename,
                       const char* name);

/* function to query a directory and retrieve an array of all .playlist
 * files in that directory */
int playlist_get_playlist_filenames(const char *dirname, char ***files);

/* functions to initalize, save, delete, and rename playlists */
playlist *playlist_load(meta_info_db *midb, const char *filename);
void playlist_save(const playlist *p);
void playlist_delete(playlist *p);

/* function to remove a file from a playlist */
void playlist_remove_file(playlist *p, int index);

#endif
