#ifndef UINTERFACE_H
#define UINTERFACE_H

#include <sys/ioctl.h>
#include <sys/ioctl_compat.h>
#include <ncurses.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>
#include <err.h>


/* return codes used for various things */
#define  UI_OK                0
#define  UI_USER_CANCELLED    1

typedef enum
{  UI_LEFT,
   UI_RIGHT
} Alignment;

typedef struct
{
   WINDOW  *cwin;             /* actual curses window */

   uint16_t width,  height;   /* width & height of curses window */


   int      max_rows;         /* max number of rows of CONTENT in the window */
   int      cur_row;          /* currently selected row in the window */
   int      voffset, hoffset; /* vertical & horizontal scroll offsets */

   /* these row-format strings (left and right aligned versions) are used
    * when you want to draw text in a window that takes up the whole
    * width of the window */
   char    *row_fmt_l;
   char    *row_fmt_r;

   int      attr_default;     /* default text attributes for all text */
   int      attr_cur_row;     /* text attributes for current row */
   int      attr_cur_play;    /* text attributes for currently playing row */

} window;


/* enumeration of the various windows that can be focused */
typedef enum
{  UI_PLAYLISTS,
   UI_MEDIALIST,
   UI_COMMAND
} WindowFocus;


/* struct used to wrap all the details of an interface together */
typedef struct
{
   /* the 4 windows used by vitunes */
   window   *player;
   window   *playlists;
   window   *medialist;
   window   *command;

   /* used to track which window is currently in focus and what the previously
    * focused window was */
   WindowFocus previous_window;
   WindowFocus active_window;

   /* record keeping stuff */
   int height;             /* height of the screen */
   int width;              /* width of the screen */
   int playlist_width;     /* width of playlists window */

   /* ui-wide attributes */
   int   attr_error;       /* text attributes for all errors */
   int   attr_bars;        /* text attributes for the bars */

} uinterface;




/* ui creation/destruction functions */
void ui_init(uinterface *ui, int playlist_width);
void ui_destroy(uinterface *ui);
void ui_resize(uinterface *ui);

/* window creation/destruction functions */
void ui_window_init(window *w, int width, int height, int startx, int starty);
void ui_window_resize(window *w, int new_width, int new_height);
void ui_window_destroy(window *w);

/* ui-wide functions */
void ui_set_focus(uinterface *ui, WindowFocus new_win);
void ui_draw_borders(uinterface *ui);
void ui_repaint_all(uinterface *ui);

/* generic window functions */
void ui_window_change_row(window *w, int diff);
void ui_window_vscroll(window *w, int diff);
void ui_window_hscroll(window *w, int diff);

/* input methods */
int  ui_command_get_string(uinterface *ui, const char *prompt, char **input);
int  ui_command_get_yesno(uinterface *ui, const char *prompt, int *response);
int  ui_get_number(uinterface *ui, int *num, char first);

#endif

