
#include "viui.h"

const char *fmtCommandInfo(const uinterface *ui)
{
   static char scratchpad[255];

   window *w = NULL;
   char   *focusName = "";
   int     percent;

   /* determine focus'd window name */
   switch (ui->active_window)
   {  case UI_PLAYLISTS:
         focusName = "playlist";
         w = ui->playlists;
         break;
      case UI_MEDIALIST:
         focusName = "medialist";
         w = ui->medialist;
         break;
      default:
         errx(1, "invalid focus reached in status info");
         break;
   }

   /* determine how far we've scrolled through the window */
   if (w->max_rows == 0)
      percent = 100;
   else
      percent = 100 * (w->voffset + w->cur_row + 1) / w->max_rows;

   sprintf(scratchpad, "[%s] %6d,%-3d %3d%%",
      focusName,
      w->voffset + w->cur_row + 1,
      w->hoffset,
      percent);

   return scratchpad;
}

const char *fmtTime4Medialist(uint64_t plength_ms)
{
   static char scratchpad[255];

   int   total_seconds = plength_ms / 1000000;
   int   seconds = total_seconds % 60;
   int   minutes = (total_seconds / 60) % 60;
   int   hours   = total_seconds / 3600;

   if (hours > 0)
      sprintf(scratchpad, "%d:%2.2d:%2.2d", hours, minutes, seconds);
   else if (minutes > 0)
      sprintf(scratchpad, "%d:%2.2d", minutes, seconds);
   else if (seconds > 0)
      sprintf(scratchpad, "%ds", seconds);
   else
      sprintf(scratchpad, " ");

   return scratchpad;
}


const char *fmtFile4Medialist(const meta_info *mi)
{
   static char scratchpad[255];

   bool has_info = false;

   /* see if the file has any meta info */
   int i;
   for (i = 0; i < MI_NUM_CINFO; i++)
   {  if (mi->cinfo[i] != NULL)
         has_info = true;
   }

   if (!has_info)
      sprintf(scratchpad, "%s", mi->file);
   else
   {  sprintf(scratchpad, "%-18.18s %-18.18s %-18.18s %-3.3s %-4.4s %9.8s",
         (mi->cinfo[MI_CINFO_ARTIST] == NULL ? "" : mi->cinfo[MI_CINFO_ARTIST]),
         (mi->cinfo[MI_CINFO_ALBUM]  == NULL ? "" : mi->cinfo[MI_CINFO_ALBUM]),
         (mi->cinfo[MI_CINFO_TITLE]  == NULL ? "" : mi->cinfo[MI_CINFO_TITLE]),
         (mi->cinfo[MI_CINFO_TRACK]  == NULL ? "" : mi->cinfo[MI_CINFO_TRACK]),
         (mi->cinfo[MI_CINFO_YEAR]   == NULL ? "" : mi->cinfo[MI_CINFO_YEAR]),
         fmtTime4Medialist(mi->plength_ms)
      );
   }

   return scratchpad;
}

const char *fmtFile4Player(const meta_info *mi)
{
   static int    cinfos[] = { MI_CINFO_ARTIST, MI_CINFO_ALBUM,
                              MI_CINFO_TITLE };
   static int    cinfo2show = 0;
   static time_t cinfoUpdated = 0;

   /* determine which cinfo item to show */
   if (time(NULL) - cinfoUpdated >= 3)
   {  cinfoUpdated = time(NULL);
      cinfo2show++;

      if (cinfo2show >= 3)
         cinfo2show = 0;
   }

   if (mi->cinfo[cinfos[cinfo2show]] != NULL)
      return mi->cinfo[cinfos[cinfo2show]];
   else
   {  int offset = 0;
      if (strlen(mi->file) > 49)
         offset += strlen(mi->file) - 49;

      return mi->file + offset;
   }

}


void viui_draw_row(window *w, int row, Alignment a, const char *str)
{
   int offset;
   if (w->hoffset < strlen(str))
      offset = w->hoffset;
   else
      offset = strlen(str);

   if (a == UI_LEFT)
      mvwprintw(w->cwin, row, 0, w->row_fmt_l, str + offset);
   else
      mvwprintw(w->cwin, row, 0, w->row_fmt_r, str + offset);
}

void viui_draw_playlists(vitunes *v)
{
   window *win = v->ui->playlists;
   wattron(win->cwin, win->attr_default);

   int row, pindex;
   for (row = 0; row < win->height; row++)
   {
      /* determine what to draw in this row of the window */
      const char *str2draw = "";
      pindex = win->voffset + row;
      if (pindex < v->library->playlists_num)
         str2draw = v->library->playlists[pindex]->name;

      /* apply text attributes: if current row is currently selected */
      if (row == win->cur_row)
         wattron(win->cwin, win->attr_cur_row);

      /* apply text attributes: if currently playing this playlist */
      if (pindex == v->playing_playlist)
         wattron(win->cwin, win->attr_cur_play);

      /* draw the row */
      viui_draw_row(win, row, UI_LEFT, str2draw);

      /* UNapply text attributes: if currently playing this playlist */
      if (pindex == v->playing_playlist)
      {  wattroff(win->cwin, win->attr_cur_play);
         wattron(win->cwin,  win->attr_default);
      }

      /* UNapply text attributes: if current row is currently selected */
      if (row == win->cur_row)
      {  wattroff(win->cwin, win->attr_cur_row);
         wattron(win->cwin, win->attr_default);
      }
   }

   wattroff(win->cwin, win->attr_default);
   wrefresh(win->cwin);
}

void viui_draw_medialist(vitunes *v)
{
   int       viewingPindex = v->viewing_playlist;
   int       playingPindex = v->playing_playlist;
   int       playingFindex = v->mplayer->current;
   window   *win   = v->ui->medialist;
   playlist *plist = v->library->playlists[viewingPindex];

   wattron(win->cwin, win->attr_default);

   int row;
   for (row = 0; row < win->height; row++)
   {
      /* determine what to draw in this row of the window */
      const char *todraw = "";
      int findex = row + win->voffset;
      if (findex < plist->num_files)
         todraw = fmtFile4Medialist(plist->files[findex]);

      /* apply text attributes: if current row is currently selected */
      if (row == win->cur_row)
         wattron(win->cwin, win->attr_cur_row);

      /* apply text attributes: if currently playing this file */
      if (playingPindex == viewingPindex && findex == playingFindex)
         wattron(win->cwin, win->attr_cur_play);

      /* draw the row */
      viui_draw_row(win, row, UI_LEFT, todraw);

      /* UNapply text attributes: if currently playing this playlist */
      if (playingPindex == viewingPindex && findex == playingFindex)
      {  wattroff(win->cwin, win->attr_cur_play);
         wattron(win->cwin,  win->attr_default);
      }

      /* UNapply text attributes: if current row is currently selected */
      if (row == win->cur_row)
      {  wattroff(win->cwin, win->attr_cur_row);
         wattron(win->cwin, win->attr_default);
      }
   }

   wattroff(win->cwin, win->attr_default);
   wrefresh(win->cwin);
}

void viui_draw_player(vitunes *v)
{
   static char  scratchpad[255];
   static char *playmode;
   static int   in_hour;
   static int   in_minute;
   static int   in_second;

   window      *win = v->ui->player;
   wattron(win->cwin, win->attr_default);

   if (!mp_status.is_playing)
      viui_draw_row(win, 0, UI_RIGHT, "...vitunes");
   else
   {  /* determine time into current selection */
      in_hour     = (int)  roundf(mp_status.position / 3600);
      in_minute   = ((int) roundf(mp_status.position)) % 3600 / 60;
      in_second   = ((int) roundf(mp_status.position)) % 60;

      /* get character for playmode */
      switch (v->mplayer->mode)
      {  case MP_MODE_LINEAR:
            playmode = "-";
            break;
         case MP_MODE_LOOP:
            playmode = "O";
            break;
         case MP_MODE_RANDOM:
            playmode = "?";
            break;
      }

      /* determine info about song to show */
      playlist *plist = v->library->playlists[v->playing_playlist];
      const char *fileInfo = fmtFile4Player(plist->files[v->mplayer->current]);

      int percent = -1;
      if (plist->files[v->mplayer->current]->plength_ms > 0)
      {  int whole = plist->files[v->mplayer->current]->plength_ms / 1000000;
         percent = roundf(100.0 * mp_status.position / whole);
      }

      /* draw it! */
      sprintf(scratchpad, "%s %8.8s +%2.2d:%2.2d:%2.2d (%3d%%) %-49.49s",
         playmode,
         (v->mplayer->paused ? "-PAUSED-" : ""),
         in_hour, in_minute, in_second,
         percent,
         fileInfo);

      viui_draw_row(v->ui->player, 0, UI_LEFT, scratchpad);
   }

   wattroff(v->ui->player->cwin, v->ui->player->attr_default);
   wrefresh(v->ui->player->cwin);

   static int currently_playing = -1;
   if (v->mplayer->current != currently_playing)
   {  currently_playing = v->mplayer->current;
      viui_draw_medialist(v);
   }
}

void viui_draw_command(vitunes *v)
{
   window *win = v->ui->command;

   wattron(win->cwin, win->attr_default);
   viui_draw_row(win, 0, UI_RIGHT, fmtCommandInfo(v->ui));
   wattroff(win->cwin, win->attr_default);

   wrefresh(win->cwin);
}

void viui_draw_command_mesg(vitunes *v, char *fmt, ...)
{
   va_list ap;

   /* clear existing */
   viui_draw_row(v->ui->command, 0, UI_LEFT, "");

   window *win = v->ui->command;
   wattron(win->cwin, v->ui->command->attr_default);

   wmove(win->cwin, 0, 0);
   va_start(ap, fmt);
   vwprintw(win->cwin, fmt, ap);
   va_end(ap);

   wattroff(win->cwin, v->ui->command->attr_default);
   wrefresh(win->cwin);
}

void viui_draw_command_error(vitunes *v, char *fmt, ...)
{
   va_list ap;

   /* clear existing */
   viui_draw_row(v->ui->command, 0, UI_LEFT, "");

   window *win = v->ui->command;
   wattron(win->cwin, v->ui->attr_error);

   wmove(win->cwin, 0, 0);
   va_start(ap, fmt);
   vwprintw(win->cwin, fmt, ap);
   va_end(ap);

   wattroff(win->cwin, v->ui->attr_error);
   beep();
   wrefresh(win->cwin);
}

